%% Figure 1 from Robust Blind Calibration via Total Least Squares
%
% Authors: John Lipor and Laura Balzano
%
% Error in gain estimation as a function of subspace error, where n = 100,
% r = 20. The dashed line represents uncalibrated error.

clear

n = 100;    % number of sensors
r = 20;     % rank of underlying subspace
MC = 100;   % number of Monte Carlo trials
I = eye(n);

varvals = [0 0.01 0.015 0.02 0.03 0.04 0.05];   % noise variances
nn = length(varvals);

svd_err = zeros(MC,nn);
ls_err = zeros(MC,nn);
pbls_err = zeros(MC,nn);
tls_err = zeros(MC,nn);
pbtls_err = zeros(MC,nn);
ss_err = zeros(MC,nn);
guess_err = zeros(MC,nn);

% loop over noise variance values
for vv = 1:nn
    
    varn = varvals(vv);
    
    % loop over Monte Carlo trials
    for mc = 1:MC
        
        % create projection matrix
        S = randn(n);
        [u,~,~] = svd(S);
        u = u(:,1:r);
        S = u*u';   % true subspace
        P = I - S;  % orthocomplement of subspace
        U = orth(S);
        V = orth(P);
        epsilon = varn*randn(size(V));
        P_est = orth(V + epsilon)*orth(V + epsilon)';
        S_est = I - P_est;
        
        % simulate actual data from this subspace
        K = ceil(3*r*log(n));
        X = randn(n,K);
        err = zeros(K,1);
        for k = 1:K
            X(:,k) = S*X(:,k);
            err(k) = norm(X(:,k) - S_est*X(:,k))/norm(X(:,k));
        end
        
        % calibration parameters
        alpha = 1 - 0.5*(rand(n,1) - 0.5);
        alpha(1) = 1;
        beta = rand(n,1) - 0.5;
        
        % observations
        Y = zeros(n,K);
        for k = 1:K
            Y(:,k) = (X(:,k) - beta)./alpha;
        end
        Ybar = diag(mean(Y,2));
        
        % observation matrix from Eq. (4) and (9)
        C = zeros(K*n,n);
        C_0 = zeros(K*n,n);
        for k = 1:K
            C((k-1)*n+1:k*n,:) = P_est*(diag(Y(:,k))-Ybar);
            C_0((k-1)*n+1:k*n,:) = P*(diag(Y(:,k))-Ybar);
        end
        
        % SVD calibration
        [~,~,v] = svd(C,0);
        ea_svd = v(:,end);
        ea_svd = ea_svd*alpha(1)/ea_svd(1);
        
        % LS calibration
        b = C(:,1)*alpha(1);
        A = C(:,2:end);
        ea_ls = A\(-b);
        ea_ls = [alpha(1); ea_ls];
                
        % PB-LS calibration
        g = 5;
        U_est = orth(S_est);
        [T,inds] = findT(U_est);    % find linearly independent columns
        i1 = inds(1);
        inds(inds == 1) = i1;
        inds(1) = 1;        
        A_tilde = C(:,inds(g+1:end));
        b_tilde = C(:,inds(1:g))*alpha(inds(1:g));
        ea_pbls = zeros(n,1);
        ea_pbls(inds(1:g)) = alpha(inds(1:g));
        ea_pbls(inds(g+1:end)) = A_tilde\-b_tilde;
        
        % PB-TLS calibration
        D = eye(n-g+1);     % scaling matrix
        D(end,end) = 1/sqrt(sum(alpha(inds(1:g))));
        [~,~,v] = svd([A_tilde b_tilde]*D,0);
        v1 = v(:,end);
        v1 = D(1:n-g,1:n-g)*v1(1:end-1)/(D(end,end)*v1(end));
        ea_pbtls = zeros(n,1);
        ea_pbtls(inds(1:g)) = alpha(inds(1:g));
        ea_pbtls(inds(g+1:end)) = v1;
        
        % TLS calibration
        g = 1;
        inds = 1:n;
        A = C(:,inds(g+1:end));
        b = C(:,inds(1:g))*alpha(inds(1:g));
        D = eye(n-g+1);     % scaling matrix
        D(end,end) = 1/sqrt(sum(alpha(inds(1:g))));
        [u,~,v] = svd([A b]*D,0);
        v1 = v(:,end);
        v1 = D(1:n-g,1:n-g)*v1(1:end-1)/(D(end,end)*v1(end));
        ea_tls = zeros(n,1);
        ea_tls(inds(1:g)) = alpha(inds(1:g));
        ea_tls(inds(g+1:end)) = v1;
        
        % errors and things to plot
        svd_err(mc,vv) = norm(alpha-ea_svd)/norm(alpha);
        ls_err(mc,vv) = norm(alpha-ea_ls)/norm(alpha);
        pbls_err(mc,vv) = norm(alpha-ea_pbls)/norm(alpha);
        tls_err(mc,vv) = norm(alpha-ea_tls)/norm(alpha);
        pbtls_err(mc,vv) = norm(alpha-ea_pbtls)/norm(alpha);
        guess_err(mc,vv) = norm(alpha-ones(n,1))/norm(alpha);
        ss_err(mc,vv) = mean(err);
        
    end
    
end

% average over Monte Carlo trials
svd_err_tot = mean(svd_err,1);
ls_err_tot = mean(ls_err,1);
pbls_err_tot = mean(pbls_err,1);
tls_err_tot = mean(tls_err,1);
pbtls_err_tot = mean(pbtls_err,1);
ss_err_tot = mean(ss_err,1);
guess_err_tot = mean(guess_err,1);

%% Plots

s = {};
figure
hold all
plot(ss_err_tot,ls_err_tot,'-o'), s{end+1} = 'LS';
plot(ss_err_tot,pbls_err_tot), s{end+1} = 'PB-LS, 5 known';
plot(ss_err_tot,svd_err_tot,'-x','MarkerSize',12), s{end+1} = 'SVD';
plot(ss_err_tot,tls_err_tot), s{end+1} = 'TLS';
plot(ss_err_tot,pbtls_err_tot), s{end+1} = 'PB-TLS, 5 known';
plot(ss_err_tot,guess_err_tot,'k--');
xlabel('subspace error')
ylabel('estimation error')
legend(s,'Location','NorthWest')
axis tight

box on
set(gcf,'Color','w')