%% Figure 3 from Robust Blind Calibration via Total Least Squares
%
% Authors: John Lipor and Laura Balzano
%
% Estimation error as a function of increasing number of known sensor gains
% and subspace error for PB-LS and PB-TLS methods. Parameters are n = 100,
% r = 20.

clear

n = 100;   % number of sensors
r = 20;    % rank of underlying subspace
MC = 50;   % number of Monte Carlo trials
I = eye(n);

varvals = [0 0.01 0.015 0.02 0.03 0.04 0.05 0.07 0.09 0.11];    % noise variances
gvals = 1:n/2;                                                  % number known gains

pbls_err = zeros(MC,length(varvals),length(gvals));
pbtls_err = zeros(MC,length(varvals),length(gvals));
ss_err = zeros(MC,length(varvals),length(gvals));

% loop over variances
for vv = 1:length(varvals)
    
    varn = varvals(vv);
    
    % loop over number of known gains
    for gg = 1:length(gvals)
        g = gvals(gg);
        
        % loop over Monte Carlo trials
        for mc = 1:MC
            
            % create projection matrix
            S = randn(n);
            [u,~,~] = svd(S);
            u = u(:,1:r);
            S = u*u';   % true subspace
            P = I - S;  % orthocomplement of subspace
            U = orth(S);
            V = orth(P);
            epsilon = varn*randn(size(V));
            P_est = orth(V + epsilon)*orth(V + epsilon)';
            S_est = I - P_est;
            
            % simulate actual data from this subspace
            K = ceil(3*r*log(n));
            X = randn(n,K);
            err = zeros(K,1);
            for k = 1:K
                X(:,k) = S*X(:,k);
                err(k) = norm(X(:,k) - S_est*X(:,k))/norm(X(:,k));
            end
            
            % calibration parameters
            alpha = 1 - 0.5*(rand(n,1) - 0.5);
            alpha(1) = 1;
            beta = rand(n,1) - 0.5;
            
            % observations
            Y = zeros(n,K);
            for k = 1:K
                Y(:,k) = (X(:,k) - beta)./alpha;
            end
            Ybar = diag(mean(Y,2));
            
            % observation matrix from Eq. (4) and (9)
            C = zeros(K*n,n);
            C_0 = zeros(K*n,n);
            for k = 1:K
                C((k-1)*n+1:k*n,:) = P_est*(diag(Y(:,k))-Ybar);
                C_0((k-1)*n+1:k*n,:) = P*(diag(Y(:,k))-Ybar);
            end
            
            % PB-LS calibration
            U_est = orth(S_est);
            [T,inds] = findT(U_est);    % find linearly independent columns
            i1 = inds(1);
            inds(inds == 1) = i1;
            inds(1) = 1;
            A_tilde = C(:,inds(g+1:end));
            b_tilde = C(:,inds(1:g))*alpha(inds(1:g));
            ea_pbls = zeros(n,1);
            ea_pbls(inds(1:g)) = alpha(inds(1:g));
            ea_pbls(inds(g+1:end)) = A_tilde\-b_tilde;
            
            % PB-TLS calibration
            D = eye(n-g+1);     % scaling matrix
            D(end,end) = 1/sqrt(sum(alpha(inds(1:g))));
            [~,~,v] = svd([A_tilde b_tilde]*D,0);
            v1 = v(:,end);
            v1 = D(1:n-g,1:n-g)*v1(1:end-1)/(D(end,end)*v1(end));
            ea_pbtls = zeros(n,1);
            ea_pbtls(inds(1:g)) = alpha(inds(1:g));
            ea_pbtls(inds(g+1:end)) = v1;
            
            % errors and things to plot
            pbls_err(mc,vv,gg) = norm(alpha-ea_pbls)/norm(alpha);
            pbtls_err(mc,vv,gg) = norm(alpha-ea_pbtls)/norm(alpha);
            ss_err(mc,vv,gg) = mean(err);
            
        end     
    end
end

% average over Monte Carlo trials
pbls_err_tot = squeeze(mean(pbls_err,1))';
pbtls_err_tot = squeeze(mean(pbtls_err,1))';
ss_err_tot = mean(squeeze(mean(ss_err,1)),2);

%% Plots

figure, imagesc(ss_err_tot,gvals,pbls_err_tot)
xlabel('subspace error','FontSize',20)
ylabel('known gains','FontSize',20)
title('PB-LS','FontSize',24)
caxis([0 1])
colorbar
box on
set(gcf,'Color','w')

figure, imagesc(ss_err_tot,gvals,pbtls_err_tot)
xlabel('subspace error','FontSize',20)
ylabel('known gains','FontSize',20)
title('PB-TLS','FontSize',24)
caxis([0 1])
colorbar
box on
set(gcf,'Color','w')